; GW49_6_ASM on H160 & WIN1_ass
; 13th January 2005
;
;		    GW_DJW1_ASM
;		    ~~~~~~~~~~
;   Routines added to GWASS assembler by Dave Walker
;
;   AMENDMENT HISTORY
;   ~~~~~~~~~~~~~~~~~
;   14 Sep 1996 DJW   - First version.
;   25 Sep 1996 GDG   - Second version - TK2_BUFFER[X] in DATA SPACE
;				   - More defaults
; 16th March 1998	GDG   - More defaults
; 20th March 1998	GDG   - Take values of DATAD$ and PROGD$ from
;			DATAD(A6) and PROGD(A6) as set by SET_DP
; 7th July 1998	GDG   - Yet another default PTR or no?
; 20th January 2001 GDG   - Correct Config Blocks for PC addressing
;					    Effective addresses
;					    COMMON sections
; 19th February 2003	GDG - Added default DEF_EXP for expressions to
;			Config block
;
; 23rd August 2004	GDG - Added DEF_SB1 and DEF_SB2 as substitutes
;			for | and ~ in macros
;
; 20th October 2004 GDG - Added DEF_PAR for choice of Qmac/GWASS pars
;
; 13th January 2005 GDG - Altered defaults: sym file "win1_ass_gass_equ3_sym"
;				  : Use pointer
;				  : Page width 120
;======================================================================
;		    STRING HANDLING
;
;   Useful string handling routines.
;
;   - These are based on C68 C style routines adapted to handle QL strings.
;   - Parameters are passed on the stack.
;   - All registers are preserved.
;   - The calling routines must tidy the stack.
;----------------------------------------------------------------------

;   Copy a QL string
;	4(A7) = Target
;	8(A7) = Source

QSTRCPY
	MOVEM.L	D0/A0-A1,-(A7)		  ; save regosters used
	MOVE.L	12+4(A7),A1		; Target address
	MOVE.L	12+8(A7),A0		; Source address
	MOVE.W	(A0),D0		; Length to copy
	MOVE.W	(A0)+,(A1)+		; Copy length field
	BRA	QSTRCPY2		; Remove if NULL byte at end wanted
QSTRCPY1	MOVE.B	(A0)+,(A1)+		; Copy a byte
QSTRCPY2	DBRA	D0,QSTRCPY1		; loop until finished
	MOVEM.L	(A7)+,D0/A0-A1		  ; restore saved registers
	RTS

;   Concatenate a QL string onto another one
;	4(A7) = Target
;	8(A7) = Source

QSTRCAT
	MOVEM.L	D0/A0-A1,-(A7)
	MOVE.L	12+4(A7),A1		; Target address
	MOVE.L	12+8(A7),A0		; Source address
	MOVE.W	(A1),D0		; get old length
	ADD.W	(A0),D0		; Calculate new length
	MOVE.W	D0,(A1)+			; Store new length
	SUB.W	(A0),D0		; Reset to old length
	ADDA.W	D0,A1			; skip over current data
	MOVE.W	(A0)+,D0			; get length to copy
	BRA	QSTRCPY2


;   Copy a QL string with maximum length check
;	4(A7).L  = Target
;	8(A7).L  = Source
;	12(A7).W = Maximum length

QSTRNCPY
	MOVEM.L	D0-D1/A0-A2,-(A7)
	MOVE.L	20+4(A7),A1	; Target address
	MOVE.L	A1,A2		; ... copied to A2
	MOVE.L	20+8(A7),A0	; Source address
	MOVE.W	20+12(A7),D1	; Maximum length
	MOVE.W	(A0)+,D0		; Length to copy
	CLR.W	(A1)+		; Clear target length
	BRA	QSTRNCPY2
QSTRNCPY1 CMP.W	(A2),D1		; Check max length not reached
	BEQ	QSTRNCPY3 	; If so exit immediately
	ADDQ.W	#1,(A2)		; Update length
	MOVE.B	(A0)+,(A1)+	; ... and copy byte
QSTRNCPY2 DBRA	D0,QSTRNCPY1	; Loop until finished
QSTRNCPY3 MOVEM.L	(A7)+,D0-D1/A0-A2
	RTS

;   Concatenate a QL string onto another one with maximum length check
;	4(A7) = Target
;	8(A7) = Source

QSTRNCAT
	MOVEM.L	D0-D1/A0-A2,-(A7)
	MOVE.L	20+4(A7),A1	; Target address
	MOVE.L	A1,A2		; copied to A2
	MOVE.L	20+8(A7),A0	; Source address
	MOVE.W	20+12(A7),D1	; Maximum length
	MOVE.W	(A1)+,D0		; Existing length
	ADDA.W	D0,A1		; Skip over source length
	MOVE.W	(A0)+,D0		; Length to copy
	BRA	QSTRNCPY2 	; Join QSTRNCPY code

;======================================================================
;		       FILE HANDLING
;
;   Friendly versions of file open/delete that will
;   allow use of TK2 directories in names.
;
;   These routines assume that the parameters have been set up
;   for the respective TRAP #2 call.  They can then be called
;   in place of the TRAP #2 routine.
;-----------------------------------------------------------------------

;   When we OPEN a file we want to try the open in
;   the following sequence of events:
;     - The name exactly as supplied
;     - If that fails, the name with the DATA_USE directory
;	added to the front.
;     - If that fails, and we are doing a read the name with
;	the PROG_USE added to the front.  We do not do this
;	with WRITE types of open as we only want those in the
;	current directory if a path is not explicitly specified.

OPENFILE
	MOVEQ	#IO_OPEN,D0
	MOVEQ	#-1,D1
	MOVEM.L	D0/A0,-(A7)	; Save name pointer

;	We start by trying the name exactly as passed

	TRAP	#2		; Try operation
	TST.L	D0		; OK?
	BEQ	OPENEXIT		; YES, jump

;	If we failed with the first try, we now want
;	to try with the DATA_USE directory added

	MOVEM.L	(A7),D0/A0	; restore saved registers
	BSR	DATA_USE
	TRAP	#2
	TST.L	D0
	BEQ	OPENEXIT

;	If the DATA_USE did not work and we are
;	attempting to read a file, then try the
;	PROG_USE as well.

	CMPI.B	#1,D3		; OPEN_IN ?
	BGT	OPENEXIT		; ... NO, then give up
	MOVEM.L	(A7),D0/A0	; restore saved registers
	BSR	PROG_USE
	TRAP	#2

OPENEXIT	ADDA.W	#2*4,A7		; remove saved values
	TST.L	D0		; set condition code
	RTS

;   When we DELETE a file we want to try the delete in
;   the following sequence of events:
;     - The name exactly as supplied
;     - If that fails, the name with the DATA_USE directory
;	added to the front.

DELETEFILE
	MOVEQ	#IO_DELET,D0
	MOVEQ	#-1,D1
	MOVEM.L	D0/D1/A0,-(A7)	; Save registers

;	We start by trying the name exactly as passed

	TRAP	#2		; Try operation
	TST.L	D0
	BEQ	DELEXIT

;	If we failed with the first try, we now want
;	to try with the DATA_USE directory added

	MOVEM.L	(A7),D0/D1/A0	; restore saved registers
	BSR	DATA_USE
	TRAP	#2

DELEXIT	ADDA.W	#3*4,A7		; remove saved values
	TST.L	D0		; set condition code
	RTS



;	Get the DATA_USE value, and build up a name
;	which has this at the front followed by the
;	users name and try that.

DATA_USE	PEA	DATAD(A6)
DATA_USE1 PEA	TK2_BUFFER(A6)	; Target
	BSR	QSTRCPY
	ADDQ.L	#8,A7		; Remove parameters
	MOVE.W	#50,-(A7) 	; Max length
	MOVE.L	A0,-(A7)		; Source
	PEA	TK2_BUFFER(A6)	; Target
	BSR	QSTRNCAT
	ADDA.W	#10,A7		; Remove parameters
	LEA	TK2_BUFFER(A6),A0	; set A0 to new filename
	RTS

;	Get the PROG_USE value, and build up a name
;	which has this at the front followed by the
;	users name and try that.

PROG_USE	PEA	PROGD(A6)
	BRA	DATA_USE1
;
; At entry D0.B = 0 for set both DATAD(A6) and PROGD(A6)
;	      = 1 for set DATAD(A6) and zero PROGD(A6)
; No registers are used
;
SET_DP	MOVEM.L	D0-2/D4/A0-1,-(A7)
	MOVE.B	D0,D4
	MOVEQ	#MT_INF,D0
	TRAP	#1
	MOVE.L	SV_DATA(A0),-(A7)
	BEQ	SET_DP1 no DATAD$
	PEA	DATAD(A6)
	BSR	QSTRCPY
	ADDQ.L	#8,A7
	BRA	SET_DP2
SET_DP1	ADDQ.L	#4,A7
	CLR.W	DATAD(A6)
SET_DP2	TST.B	D4
	BNE	SET_DP3		zero PROGD
	MOVE.L	SV_PROG(A0),-(A7)
	BEQ	SET_DP4		no PROGD$
	PEA	PROGD(A6)
	BSR	QSTRCPY
	ADDQ.L	#8,A7
	BRA	SET_DP5
SET_DP4	ADDQ.L	#4,A7
SET_DP3	CLR.W	PROGD(A6)
SET_DP5	MOVEM.L	(A7)+,D0-2/D4/A0-1
	RTS

;=========================================================================
;			CONFIG BLOCK
;
;   This sets up a CONFIG block for options that the user might want
;   to pre-set.  The aim is to avoid having to have different
;   versions of this program for different defaults.
;-------------------------------------------------------------------------

; Config ID
	DC.L	'<<QC','FX>>'	    ; Configuration ID
	DC.W	'01'		    ; Configuration level
CFGNAME	DC.W	CFGNAMEX-CFGNAME-2	    ; Software name
	DC.B	'GWASS Assembler'
CFGNAMEX	DS.B	0
CFGVER	DC.W	CFGVERX-CFGVER-2	    ; Software Version
	DC.B	VERSION
CFGVERX	DS.B	0

item1	DS.W	0		    ; Word Align

	DC.B	0		    ; Item type  (0 = string)
	DC.B	'A'		    ; Item shortcut key
	DC.W	EQU_SYM_MAX-*	    ; Item pointer
	DC.W	0		    ; Item pre-processing routine
	DC.W	0		    ; Item post-processing routine
	DC.W	DESCEQU-* 	    ; Item description pointer
	DC.W	STRATT-*		    ; item attributes pointer

item2	DS.W	0		    ; Word align

	DC.B	0
	DC.B	'D'
	DC.W	FILEDEV_MAX-*
	DC.W	0
	DC.W	0
	DC.W	DESCDEV-*
	DC.W	STRATT-*

ITEM3	DC.B	4,0
	DC.W	DEF_CD-*,0,0
	DC.W	DESCD-*
	DC.W	ATT_CD-*

ITEM4	DC.B	4,0
	DC.W	DEF_68-*,0,0
	DC.W	DES68-*
	DC.W	ATT_68-*

ITEM5	DC.B	4,0
	DC.W	DEF_EL-*,0,0
	DC.W	DESEL-*
	DC.W	ATT_EL-*

ITEM6	DC.B	4,0
	DC.W	DEF_UX-*,0,0
	DC.W	DESUX-*
	DC.W	ATT_UX-*

ITEM7	DC.B	4,0
	DC.W	DEF_PC-*,0,0
	DC.W	DESPC-*
	DC.W	ATT_PC-*

ITEM8	DC.B	4,0
	DC.W	DEF_EA-*,0,0
	DC.W	DESEA-*
	DC.W	ATT_EA-*

ITEM9	DC.B	10,0	WORD
	DC.W	DEF_PCK-*,0,0
	DC.W	DESPCK-*
	DC.W	ATTPCK-*

ITEM10	DC.B	10,0	WORD
	DC.W	DEF_PWID-*,0,0
	DC.W	DESPWID-*
	DC.W	ATTPWID-*

ITEM11	DC.B	12,0	LONG WORD
	DC.W	DEF_DATA-*,0,0
	DC.W	DESDATA-*
	DC.W	ATTDATA-*

ITEM12	DC.B	4,0
	DC.W	DEF_REL-*,0,0
	DC.W	DESREL-*
	DC.W	ATT_REL-*

ITEM13	DC.B	4,0
	DC.W	DEF_COM-*,0,0
	DC.W	DESCOM-*
	DC.W	ATT_COM-*

ITEM14	DC.B	4,0
	DC.W	DEF_PTR-*,0,0
	DC.W	DESPTR-*
	DC.W	ATT_PTR-*

ITEM15	DC.B	4,0
	DC.W	DEF_EXP-*,0,0
	DC.W	DESEXP-*
	DC.W	ATT_EXP-*

ITEM16	DC.B	8,0		byte
	DC.W	DEF_SB1-*,0,0
	DC.W	DESSB1-*
	DC.W	ATT_SB-*

ITEM17	DC.B	8,0		byte
	DC.W	DEF_SB2-*,0,0
	DC.W	DESSB2-*
	DC.W	ATT_SB-*

ITEM18	DC.B	4,0
	DC.W	DEF_PAR-*,0,0
	DC.W	DESPAR-*
	DC.W	ATT_PAR-*

term
	DC.W	-1		    ; Terminate list

;   description strings

DESCEQU	DC.W	DESCEQUX-DESCEQU-2
	DC.B	'Automatic include file'
DESCEQUX	DS.B	0

DESCDEV	DC.W	DESCDEVX-DESCDEV-2
	DC.B	'Default source device'
DESCDEVX	DS.B	0

DESCD	DC.W	DESCDE-DESCD-2
	DC.B	'Case dependence'
DESCDE	DS.B	0

DES68	DC.W	DES68E-DES68-2
	DC.B	'Type of computer'
DES68E	DS.B	0

DESEA	DC.W	DESEAE-DESEA-2
	DC.B	'Effective address'
DESEAE	DS.B	0

DESEL	DC.W	DESELE-DESEL-2
	DC.B	'Type of listing'
DESELE	DS.B	0

DESPC	DC.W	DESPCE-DESPC-2
	DC.B	'PC or absolute addressing'
DESPCE	DS.B	0

DESUX	DC.W	DESUXE-DESUX-2
	DC.B	'Undefined labels as ERRORs/XREF'
DESUXE	DS.B	0	

DESPCK	DC.W	DESPCKE-DESPCK-2
	DC.B	'Number of HEX places of PC printed'
DESPCKE	DS.B	0
DESDATA	DC.W	DESDATE-DESDATA-2
	DC.B	'Size of data space'
DESDATE	DS.B	0
DESPWID	DC.W	DESPWIDE-DESPWID-2
	DC.B	'Width of page'
DESPWIDE	DS.B	0
DESREL	DC.W	DESRELE-DESREL-2
	DC.B	'o or REL trailer'
DESRELE	DS.B	0
DESCOM	DC.W	DESCOME-DESCOM-2
	DC.B	'Type of COMMON'
DESCOME	DS.B	0
	HED	<'Whether to use a pointer or not'>,DESPTR
	HED	<'Allowing expressions'>,DESEXP
	HED	<'Allow Qmac or GWASS macro paremeters'>,DESPAR
	HED	<'Substitute character for '>,DESSB1
	HED	<'Substitute character for '>,DESSB2

;   attributes

STRATT	DC.W	0		    ; No special attributes

ATT_CD	DC.B	0,0
ATTCDS	DC.W	ATTCDSE-ATTCDS-2
	DC.B	'Case independent'
ATTCDSE	DS.B	0
	DS.W	0
	DC.B	1,0
ATTCDS1	DC.W	ATTCDS1E-ATTCDS1-2
	DC.B	'Case dependent'
ATTCDS1E	DS.B	0
	DC.W	-1


ATT_EA	DC.B	0,0
ATTEAS	DC.W	ATTEASE-ATTEAS-2
	DC.B	'All effective addresses'
ATTEASE	DS.B	0
	DS.W	0
	DC.B	1,0
ATTEAS1	DC.W	ATTEAS1E-ATTEAS1-2
	DC.B	'68000/8 effective addresses'
ATTEAS1E	DS.B	0
	DC.W	-1

ATT_EL	DC.B	0,0
ATTELS	DC.W	ATTELSE-ATTELS-2
	DC.B	'Full listing'
ATTELSE	DS.B	0
	DS.W	0
	DC.B	1,0
ATTELS1	DC.W	ATTELS1E-ATTELS1-2
	DC.B	'Only errors listed'
ATTELS1E	DS.B	0
	DC.W	-1

ATT_PC	DC.B	0,0
ATTPCS	DC.W	ATTPCSE-ATTPCS-2
	DC.B	'PC addressing'
ATTPCSE	DS.B	0
	DS.W	0
	DC.B	1,0
ATTPCS1	DC.W	ATTPCS1E-ATTPCS1-2
	DC.B	'Absolute addressing'
ATTPCS1E	DS.B	0
	DC.W	-1

ATT_68	DC.B	0,0
ATT68S	DC.W	ATT68SE-ATT68S-2
	DC.B	'68040'
ATT68SE	DS.B	0
	DS.W	0
	DC.B	1,0
ATT68S1	DC.W	ATT68S1E-ATT68S1-2
	DC.B	'68030'
ATT68S1E	DS.B	0
	DS.W	0
	DC.B	2,0
ATT68S2	DC.W	ATT68S2E-ATT68S2-2
	DC.B	'68EC030'
ATT68S2E	DS.B	0
	DS.W	0
	DC.B	3,0
ATT68S3	DC.W	ATT68S3E-ATT68S3-2
	DC.B	'68851'
ATT68S3E	DS.B	0
	DC.W	-1

ATT_UX	DC.B	0,0
ATTUXS	DC.W	ATTUXSE-ATTUXS-2
	DC.B	'ERRORs'
ATTUXSE	DS.B	0
	DS.W	0
	DC.B	1,0
ATTUXS1	DC.W	ATTUXS1E-ATTUXS1-2
	DC.B	'XREF'
ATTUXS1E	DS.B	0
	DC.W	-1

ATT_REL	DC.B	0,0
ATT_REL1	DC.W	ATT_REL1E-ATT_REL1-2
	DC.B	'o trailer'
ATT_REL1E DS.B	0
	DS.W	0
	DC.B	1,0
ATT_REL2	DC.W	ATT_REL2E-ATT_REL2-2
	DC.B	'REL trailer'
ATT_REL2E DS.B	0
	DC.W	-1

ATT_COM	DC.B	0,0
ATT_COMS	DC.W	ATT_COME-ATT_COMS-2
	DC.B	'Dummy COMMON sections'
ATT_COME	DS.B	0
	DS.W	0
	DC.B	1,0
ATT_COMS1 DC.W	ATT_COME1-ATT_COMS1-2
	DC.B	'Ordinary COMMON sections'
ATT_COME1 DS.B	0
	DC.W	-1

ATT_PTR	DC.B	0,0
	HED	<"Don't use pointer">
	DS.W	0
	DC.B	1,0
	HED	<"Use pointer">
	DC.W	-1

ATT_EXP	DC.B	0,0
	HED	<"Ban expressions">
	DS.W	0
	DC.B	1,0
	HED	<"Allow expressions">
	DC.W	-1

ATT_PAR	DC.B	0,0
	HED	<"GWASS style parameters">
	DS.W	0
	DC.B	1,0
	HED	<"Qmac style parameters">
	DC.W	-1


ATTPCK	DC.W	0,8
ATTDATA	DC.L	0,1000000
ATTPWID	DC.W	70,132
ATT_SB	DC.B	128,191

; the values and storage

EQU_SYM_MAX
	DC.W	40		      ; Maximum length allowed for
EQU_SYM	DC.W	EQU_SYME-EQU_SYM-2	      ; Current string length
	DC.B	'win1_ass_gwass_equ3_sym' ; Current text
EQU_SYME	DS.B	18

FILEDEV_MAX
	DC.W	20
FILEDEV	DC.W	FILEDEVX-FILEDEV-2
	DC.B	'RAM1_'
FILEDEVX	DS.B	16
	DS.W	0

DEF_PTR	DC.B	1
DEF_COM	DC.B	0
DEF_PC	DC.B	0
DEF_EL	DC.B	0
DEF_EA	DC.B	0
DEF_68	DC.B	0
DEF_UX	DC.B	0
DEF_CD	DC.B	0
DEF_REL	DC.B	0
DEF_EXP	DC.B	0
DEF_PAR	DC.B	0
DEF_PCK	DC.W	4
DEF_PWID	DC.W	120
DEF_DATA	DC.L	4000
DEF_SB1	DC.B	''
DEF_SB2	DC.B	''
